/*
 * Hey YO!
 * This code is by Alex Markley (CoolMan)
 * It is a basic OpenGL skeleton app for Linux/SDL
 * (And if it looks a little bit like the
 * tutorial at http://nehe.gamedev.net/
 * that's because it's based off it.) 8^D
 *
 * Thanks Jeff Molofee and Ti Leggett!
 */

#include <stdio.h>
#include <stdlib.h>

#include <math.h>

#include <string.h>

#include <unistd.h>

#include <GL/gl.h>
#include <GL/glu.h>

#include "SDL.h"
#include "SDL_thread.h"

#include <linux/fs.h>
#include <sys/types.h>
#include <dirent.h>

#include <errno.h>
extern int errno;

//DEFINES
//DEFINE: BOOL
#define TRUE (int)1
#define FALSE (int)0
//
//DEFINE: WINDOW ATTRIBUTES
#define _window_width 640
#define _window_height 480
#define _window_color_bitdepth 16
#define FULLSCREEN TRUE

//VARS
//VARS: SDL STUFF
int _sdl_video_options;
SDL_Event event;
const SDL_VideoInfo *_sdl_video_info;
SDL_Surface *_sdl_surface;
//
//VARS: BOOLS
int running = TRUE;
int finished = FALSE;
//
//VARS: MISC
int _keyboard[1024];
//GLuint TextTexture;
//
//VARS: CONTROL
int up = 0, down = 0, left = 0, right = 0;
int accept = 0, back = 0;

//SUBROUTINES
int init_all(void);
void Quit(int rcode);
void evhand(void);
int event_logic();

int main( int argc, char **argv )
{
for(_keyboard[0] = 0;_keyboard[0] < 1024;_keyboard[0]++) _keyboard[_keyboard[0]] = 0;
_keyboard[0] = 0;

if(!init_all())
	{
	printf("A fatal error occurred, quitting...\n");
	Quit(1);
	}

while(!finished)
	{
	evhand();
	event_logic();
	}

Quit(0);
return(0);
}

int init_all(void)
{
if(SDL_Init(SDL_INIT_VIDEO) < 0)
	{
	printf("FATAL:\n  Failed: SDL_Init(SDL_INIT_VIDEO)\n    String: %s", SDL_GetError());
	Quit(1);
	}

_sdl_video_info = SDL_GetVideoInfo();

if(!_sdl_video_info)
	{
	printf("FATAL:\n  Failed: _sdl_video_info = SDL_GetVideoInfo()\n    String: %s", SDL_GetError());
	}

_sdl_video_options =
	SDL_OPENGL;
	//SDL_OPENGL |
	//SDL_GL_DOUBLEBUFFER |
	//SDL_HWPALETTE;

if(FULLSCREEN) _sdl_video_options |= SDL_FULLSCREEN;

/*
if(_sdl_video_info->hw_available)
	_sdl_video_options |= SDL_HWSURFACE;
else
	_sdl_video_options |= SDL_SWSURFACE;

if(_sdl_video_info->blit_hw)
	_sdl_video_options |= SDL_HWACCEL;
*/

SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

if(!(_sdl_surface = SDL_SetVideoMode(_window_width, _window_height, _window_color_bitdepth, _sdl_video_options)))
	{
	printf("FATAL:\n  Failed: _sdl_surface = SDL_SetVideoMode(_window_width, _window_height, _window_color_bitdepth, _sdl_video_options)\n    String: %s", SDL_GetError());
	Quit(1);
	}

/* Enable smooth shading */
glShadeModel( GL_SMOOTH );

/* Set the background black */
glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );

/* Depth buffer setup */
glClearDepth( 1.0f );

/* Enables Depth Testing */
glEnable( GL_DEPTH_TEST );

/* The Type Of Depth Test To Do */
glDepthFunc( GL_LEQUAL );

/* Really Nice Perspective Calculations */
glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );

glEnable(GL_BLEND);
glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

glEnable(GL_TEXTURE_2D);

if(!_window_height)
	{
	printf("FATAL:\n  Failed: Divide by zero avoided. (This should never happen.)");
	Quit(1);
	}

glViewport(0, 0, (GLsizei)_window_width, (GLsizei)_window_height);

glMatrixMode(GL_PROJECTION);
glLoadIdentity();
gluPerspective(45.0f, (GLfloat)_window_width / (GLfloat)_window_height, 0.1f, 100.0f);

glMatrixMode(GL_MODELVIEW);
glLoadIdentity();

if(FULLSCREEN) SDL_ShowCursor(SDL_DISABLE);

return(TRUE);
}

void Quit(int rcode)
{
if(FULLSCREEN) SDL_ShowCursor(SDL_ENABLE);

printf("Will it crash?\n");
SDL_Quit(); //Destroy the Window.
printf("No! Maybe next time.\n");

exit(rcode);//Return the correct code.
}

void evhand(void)
{
while(SDL_PollEvent(&event))
	{
	switch(event.type)
		{
		case SDL_ACTIVEEVENT:
			if(event.active.gain)
				running = TRUE;
			else
				running = FALSE;
			break;
		
		case SDL_KEYDOWN:
			_keyboard[event.key.keysym.sym] = TRUE;
			//printf("INFO:\n  KeyPressed:\n    %c,%d,%X\n", event.key.keysym.sym, event.key.keysym.sym, event.key.keysym.sym);
			break;
			
		case SDL_KEYUP:
			_keyboard[event.key.keysym.sym] = FALSE;
			//printf("INFO:\n  KeyReleased:\n    %c,%d,%X\n", event.key.keysym.sym, event.key.keysym.sym, event.key.keysym.sym);
			break;

		case SDL_QUIT:
			finished = TRUE;
			break;
		
		default:
			break;
		}
	}
}

int event_logic(void)
{
if(_keyboard[(int)'q']) finished = TRUE;

if(_keyboard[SDLK_UP]) up = TRUE;
else up = FALSE;
if(_keyboard[SDLK_DOWN]) down = TRUE;
else down = FALSE;
if(_keyboard[SDLK_LEFT]) left = TRUE;
else left = FALSE;
if(_keyboard[SDLK_RIGHT]) right = TRUE;
else right = FALSE;
if(_keyboard[SDLK_RETURN]) accept = TRUE;
else accept = FALSE;
if(_keyboard[SDLK_RSHIFT]) back = TRUE;
else back = FALSE;

return TRUE;
}